@@ -1,8 +1,9 @@
 #include "rife_wrapped.h"
 
 RifeWrapped::RifeWrapped(int gpuid, bool _tta_mode, bool _uhd_mode,
-                         int _num_threads, bool _rife_v2)
-    : RIFE(gpuid, _tta_mode, _uhd_mode, _num_threads, _rife_v2)
+                         int _num_threads, bool _rife_v2, bool _rife_v4)
+    : RIFE(gpuid, _tta_mode, _uhd_mode,
+           _num_threads, _rife_v2, _rife_v4)
 {
 }
 
